import { Disposer, DisposerFunction } from '../dispose-types.cjs';
import { Emitter } from '../msg/emitter.cjs';
import '../global.cjs';
import '../log/log.cjs';
import '../log/log-base.cjs';

interface ProgressOptions {
    totalUnits?: number;
    completeUnits?: number;
    resetWhenFinished?: boolean;
    name?: string;
}
/**
 * Progress helper with these properties:
 *
 * - `totalUnits` and `completedUnits` for progress
 * - Can be cancelled
 * - Sends events on cancel, changed and dispose
 * - Supports children and propagates values.
 *   Total progress is sum of all units in the tree.
 * - On `dispose` child removes itself from parent.
 */
declare class Progress extends Emitter<{
    progressCancelled: (progress: Progress) => void;
    progressChanged: (progress: Progress) => void;
    progressDispose: (progress: Progress) => void;
}> {
    private _totalUnits;
    private _completedUnits;
    private _isCancelled;
    private _resetWhenFinished;
    private _children;
    dispose: ((strictSync?: boolean) => void | Promise<any>) & {
        disposed: number;
        add: (obj?: Disposer | undefined) => DisposerFunction | undefined;
        remove: (disposable: Disposer) => void | Promise<void>;
        track: (obj?: Disposer | undefined) => DisposerFunction | undefined;
        untrack: (disposable: Disposer) => void | Promise<void>;
        dispose: (strictSync?: boolean) => void | Promise<any>;
        disposeSync: () => void;
        sync: () => void;
        exec: (strictSync?: boolean) => void | Promise<any>;
        getSize(): number;
        isDisposed(): boolean;
    };
    name: string;
    constructor(opt?: ProgressOptions);
    /** Notify others about changes and reset on completion, if flag is set. */
    private update;
    /** Fresh start */
    reset(): void;
    /** Notify and mark as cancelled. May take some time before having an effect. */
    cancel(): Promise<void>;
    /** Add child progress, which count into its parents units. On `dispose` it will auto remove itself from parent. */
    addChild(child: Progress): void;
    /** Create child progress.  */
    createChildProgress(opt?: ProgressOptions): Progress;
    removeChild(child: Progress): void;
    /** Total units including children */
    getTotalUnits(): number;
    /** Completed units including children */
    getCompletedUnits(): number;
    /** Cannot calculate progress, because totalUnits are missing. Special representation in UI. */
    isIndeterminate(): boolean;
    isCancelled(): boolean;
    /** Either disposed or all units completed. */
    isFinished(): boolean;
    /** Value from 0 to 1, where 1 is 100% completeness. */
    getFraction(): number;
    getChildrenCount(): number;
    /** Change total units. */
    setTotalUnits(units: number, completedUnits?: number): void;
    /** Relatively change total units. */
    incTotalUnits(step?: number): void;
    /** Set fixed number of completed units. */
    setCompletetedUnits(units: number): void;
    /** Set to 100% without disposing. */
    setCompleted(): void;
    /** Dynamically change completed units. */
    incCompletedUnits(step?: number): void;
    /** Progress tree to string for debuggin purposes. Consider using `name` attribute of Progress. */
    toString(indent?: number): string;
}

export { Progress };
